from sbase import steps  # noqa
"""
This is a proxy for importing SeleniumBase-Behave steps.
A short path name shortens the output of "behave" tests.
See "../../sbase/steps.py" for the actual list of steps.
Eg.
    BEHAVE STEP NAME  # ../../sbase/steps.py:11 0.111s
    BEHAVE STEP NAME  # ../../sbase/steps.py:74 0.550s
    BEHAVE STEP NAME  # ../../sbase/steps.py:67 0.385s
"""
